﻿using System;
using System.Collections.Generic;
using System.Text;
using Eclr.Pcos;
using ProConOS_eCLR;
using RTC_APILib;
using System.Iec61131Lib;
using Eclr;

namespace RTC_Template
{
    /// <summary>
    /// ファイルアクセス処理クラス
    /// </summary>
    unsafe public class FileAccesser
    {

        #region フィールド

        private IecString80 fileName;
        private File _fp = null;

        #endregion


        #region コンストラクタ
        /// <summary>
        /// stirngパス指定コンストラクタ
        /// </summary>
        /// <param name="path"></param>
        public FileAccesser(string path)
        {
            fileName.ctor();
            fileName.s.Init(path);
            _fp = Eclr.File.GetFreeFileSlot();
        }

        #endregion


        #region プライベートメソッド

        /// <summary>
        /// オープン済み確認
        /// </summary>
        /// <returns></returns>
        private bool IsOpened()
        {
            if (_fp == null) { return false; }
            else { return true; }
        }

        /// <summary>
        /// 指定文字列から、指定サイズ分のIecString80文字列を抽出します。
        /// </summary>
        /// <param name="dst"></param>
        /// <param name="startIndex"></param>
        /// <param name="preLength"></param>
        /// <param name="pSrc"></param>
        /// <param name="beforeLength"></param>
        /// <returns></returns>
        unsafe private static int ExtractIecString(ref IecString80 dst, int startIndex, int preLength, byte* pSrc, int beforeLength)
        {
            dst = new IecString80();
            dst.ctor();
            pSrc[beforeLength] = 0;     // 終端文字へ変更
            IecStringEx.Copy(&pSrc[startIndex], ref dst.s);
            dst.s.currentLength = (short)(beforeLength - 1 - preLength);
            return preLength;
        }

        #endregion


        #region パブリックメソッド

        /// <summary>
        /// ファイルオープン
        /// </summary>
        /// <returns></returns>
        public bool Open()
        {
            if (_fp.Open(fileName.s.GetIecString(), (ushort)Eclr.FileMode.Open) == false)
            {
                if (_fp.Open(fileName.s.GetIecString(), (ushort)Eclr.FileMode.Create) == false)
                {
                    fileName.rctor();
                    _fp.IsReserved = true;
                    _fp.IsUsed = true;
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// ファイルクローズ
        /// </summary>
        /// <returns></returns>
        public bool Close()
        {
            if (_fp != null && _fp.Close())
            {
                _fp.IsReserved = false;
                _fp.IsUsed = false;
                _fp = null;
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// 指定サイズ分読み込みます。
        /// </summary>
        /// <param name="startIndex"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        unsafe public IecString80 Read(uint startIndex, uint length)
        {
            short readLength = 0;
            IecString80 readString = new IecString80();

            // バッファ初期化
            readString.ctor();
            readString.s.currentLength = 0;

            // 状態確認
            if (!IsOpened()) { return readString; }

            // 引数確認
            if (startIndex > length) { return readString; }

            // 読み込み
            readLength = (short)_fp.Read(readString.s.GetIecString(), startIndex, length);      // Offsetの意味不明（値を変化させても、結果が変わらない。）
            readString.s.currentLength = readLength;

            return readString;
        }

        /// <summary>
        /// 1行読み込み
        /// </summary>
        /// <param name="buffer"></param>
        /// <returns></returns>
        unsafe public int ReadLine(ref IecString80 buffer)
        {
            IecString80 bufOne;
            uint readsize = 0;
            uint readIndex = 0;
            int charCount = 0;

            // 状態確認
            if (!IsOpened()) { return 0; }

            // バッファの生成
            buffer = new IecString80();
            buffer.ctor();
            bufOne = new IecString80();
            bufOne.ctor();

            while (true)
            {
                // バッファ確認
                if (charCount > buffer.s.maximumLength)
                {
                    // バッファ満杯
                    break;
                }
                // 1byteずつ読み込み
                readsize = (uint)_fp.Read(bufOne.s.GetIecString(), readIndex, 1);
                if (readsize <= 0)
                {
                    // EOF
                    break;
                }
                // 読み込みインデックスの更新
                readIndex += readsize;
                if (*(bufOne.s.GetIecString()) == 0x0D)
                {
                    _fp.Read(bufOne.s.GetIecString(), readIndex, 1);
                    if (*(bufOne.s.GetIecString()) == 0x0A)
                    {
                        // 改行コードを検出したら終了
                        break;
                    }
                }
                // 読み込み文字をバッファへ格納
                buffer.s.GetIecString()[charCount++] = *(bufOne.s.GetIecString());
            }


            // 終端コード挿入
            buffer.s.GetIecString()[charCount] = 0x00;
            // 文字サイズの格納
            buffer.s.currentLength = (short)charCount;

            return buffer.s.currentLength;
        }





        /// <summary>
        /// 指定文字列を指定デリミタで分割します。
        /// </summary>
        /// <param name="csvStr"></param>
        /// <param name="split"></param>
        /// <param name="str"></param>
        /// <param name="arraySize"></param>
        /// <returns></returns>
        /// 

        unsafe public  bool Spilit(IecString80 csvStr, byte split, ref IecString80[] str, int arraySize)
        {
            int startIndex = 0;
            int dstIndex = 0;
            int preLength = 0;

            byte* pCsvStr = csvStr.s.GetIecString();
            str = new IecString80[arraySize];

            // カンマ位置/数の取得
            dstIndex = 0;
            for (int i = 0; i < csvStr.s.currentLength; i++)
            {
                //スプリット文字か？
                if (pCsvStr[i] == split)
                {
                    if (dstIndex != 0)
                    {
                        preLength += (str[dstIndex - 1].s.currentLength + 1); // 前回取得文字数 +　終端文字数
                    }
                    // 指定文字列から、指定サイズ分のIecString80文字列を抽出します。
                    preLength = ExtractIecString(ref str[dstIndex], startIndex, preLength, pCsvStr, i);
                    startIndex = (i + 1);
                    dstIndex++;
                }
            }
            preLength = 0;
            if (dstIndex != 0)
            {
                preLength += (str[dstIndex - 1].s.currentLength + 1);
            }

            // 指定文字列から、指定サイズ分のIecString80文字列を抽出します。
            preLength = ExtractIecString(ref str[dstIndex], startIndex, preLength, pCsvStr, csvStr.s.currentLength + 1);

            return true;
        }


        /// <summary>
        /// 指定文字列から指定デリミタのデータお抽出します。
        /// </summary>
        /// <param name="csvStr"></param>
        /// <param name="split"></param>
        /// <param name="GetData"></param>
        /// <param name="arraySize"></param>
        /// <param name="Point"></param>
        /// <returns></returns>
        /// 

        unsafe public  bool SpilitEx(IecString80 csvStr, byte split, ref IecString80 GetData, int arraySize,int Point)
        {
            int startIndex = 0;
            int dstIndex = 0;
            int preLength = 0;

            byte* pCsvStr = csvStr.s.GetIecString();
            IecString80[] str = new IecString80[arraySize];

            // カンマ位置/数の取得
            dstIndex = 0;
            for (int i = 0; i < csvStr.s.currentLength; i++)
            {
                //スプリット文字か？
                if (pCsvStr[i] == split)
                {
                    if (dstIndex != 0)
                    {
                        preLength += (str[dstIndex - 1].s.currentLength + 1); // 前回取得文字数 +　終端文字数
                    }
                    preLength = ExtractIecString(ref str[dstIndex], startIndex, preLength, pCsvStr, i);
                    startIndex = (i + 1);
                    dstIndex++;
                }
            }
            preLength = 0;
            if (dstIndex != 0)
            {
                preLength += (str[dstIndex - 1].s.currentLength + 1);
            }
            preLength = ExtractIecString(ref str[dstIndex], startIndex, preLength, pCsvStr, csvStr.s.currentLength + 1);

            GetData = str[Point];

            return true;
        }


        /// <summary>
        /// 文字列サイズの取得
        /// </summary>
        /// <param name="pVal"></param>
        /// <returns></returns>
        unsafe public static short GetStrLength(byte* pVal)
        {
            short cnt = 0;

            for (int i = 0; i < 80; i++)
            {
                if (*pVal == 0)
                {
                    break;
                }
                cnt++;
                pVal++;
            }

            return cnt;
        }

        #endregion

    }
}
