﻿using Eclr;
using Eclr.Pcos;
using ProConOS_eCLR;
using RTC_APILib;
using RTC_CAN_Lib;
using RTC_COMLib;
using RTC_ECAT_Lib;
using RTC_LOGLib;
using RTC_SOCKLib;
using System;
using System.Iec61131Lib;

namespace RTC_Template
{
    [FUNCTION("INT")]
    public unsafe class RTC_Program_01
    {

        public static int arraySize = 10;   //CSVデータの列数 (サンプル：CSV項目数10列)

        /// <summary>
        /// １次元の共有メモリの配列を２次元配列へ変換
        /// </summary>
        /// <param name="Arr1"></param>
        /// <param name="Arr2"></param>
        /// <param name="Array2Size"></param>
        /// <returns></returns>
        unsafe private static int Convert2Drray(int Arr1, int Arr2, int Array2Size)
        {
            return CRTC.pData[(Arr1 * Array2Size) + Arr2];
        }


        /// <summary>
        /// Task01で動作するプログラム
        /// </summary>
        /// <param name="inState">ステータス定義(STS_INI:初期処理、STS_PRO:通常処理、STS_END:終了処理、STS_ERR:エラー処理)</param>
        /// <returns></returns>
        public static Int16 __Process([VAR_INPUT("INT")] Int16 inState)
        {
            Int16 outState = 99;                // 終了時のステータス格納用変数

            switch (inState)
            {
                case CRTC.STS_INI:
                    outState = On_Init();       //初期処理
                    break;
                case CRTC.STS_PRO:
                    outState = On_Process();    //通常処理
                    break;
                case CRTC.STS_END:
                    outState = On_End();        //終了処理
                    break;
                case CRTC.STS_ERR:
                    outState = On_Err();        //エラー処理
                    break;
            }

            return outState;                    // FUの実行結果を返却
        }

        //-----------------------------------
        //■ 初期処理
        //-----------------------------------
        static short On_Init()
        {
            Int16 inReturn = CRTC.STS_INI;

            //---------------------------------------------------------
            //↓以下、ユーザが初期処理を記述します。
            //---------------------------------------------------------

            //---------------------------------
            //CSVファイル読込
            //---------------------------------

            int iLine = 0;                                      //読み込み行数
            int dataLen = 0;                                    //読み込みデータサイズ
            //int arraySize = 10;                                 //CSVデータの列数（例：10列）
            bool bAns;                                          //指定文字列を指定デリミタで分割した結果

            IecString80 buffe = new IecString80();              //ファイルデータ読み込みバッファ
            buffe.ctor();
            buffe.s.Init("");

            IecString80 sFmt = new IecString80();               //データフォーマット文字
            sFmt.ctor();
            sFmt.s.Init("%d");

            IecString80[] str = new IecString80[arraySize];     //デリミタで分割した文字列データ

            FileAccesser sMemName = new FileAccesser("C:\\Work\\RTC_Test.csv"); //読み込みファイルパスセット


            //ファイルオープン
            if (sMemName.Open() == false)
            {
                //ファイルオープンエラー
            }

            //ファイル全行読み込み
            while (true)
            {
                //1行読込
                dataLen = sMemName.ReadLine(ref buffe);
                if (dataLen <= 0)
                {
                    //読み込みデータなし
                    break;
                }

                //デリミタで分割
                bAns = sMemName.Spilit(buffe, 0x2c, ref str, arraySize);
                if (bAns)
                {
                    //分解エラー
                }

                //共有メモリへ書き込み
                for (int r = 0; r < arraySize; r++)
                {
                    //文字列から指定フォーマットに変換し共有メモリへ書き込み
                    CRTC.pData[(iLine * arraySize) + r] = Helper.StoI(str[r].s.GetIecString(), sFmt.s.GetIecString());
                }

                iLine += 1;     //読み込み行数インクリメント
            }

            //ファイルクローズ
            sMemName.Close();

            inReturn = CRTC.STS_PRO;    //通常処理へ状態移行

            //---------------------------------------------------------
            //↑ユーザが記述する初期処理はここまでです。
            //---------------------------------------------------------

            return inReturn;
        }

        //-----------------------------------
        // ■通常処理
        //-----------------------------------
        static short On_Process()
        {
            Int16 inReturn = CRTC.STS_PRO;

            //---------------------------------------------------------
            //↓以下、ユーザが通常処理を記述します。
            //---------------------------------------------------------

            //２次元配列データ取得サンプル
            //※設定・デバッグツール (MULTIPROG）のウォッチウィンドウで確認出来ます。
            CRTC.pMArea->Array_0_0 = Convert2Drray(0, 0, arraySize); //2次元配列データ[0][0]
            CRTC.pMArea->Array_9_9 = Convert2Drray(9, 9, arraySize); //2次元配列データ[9][9]


            //---------------------------------------------------------
            //↑ユーザが記述する通常処理はここまでです。
            //---------------------------------------------------------

            return inReturn;
        }

        //-----------------------------------
        // ■終了処理
        //-----------------------------------
        static short On_End()
        {
            Int16 inReturn = CRTC.STS_END;

            //---------------------------------------------------------
            //↓以下、ユーザが終了処理を記述します。
            //---------------------------------------------------------

            //---------------------------------------------------------
            //↑ユーザが記述する終了処理はここまでです。
            //---------------------------------------------------------

            return inReturn;
        }

        //-----------------------------------
        // ■エラー処理
        //-----------------------------------
        static short On_Err()
        {
            Int16 inReturn = CRTC.STS_ERR;

            //---------------------------------------------------------
            //↓以下、ユーザがエラー処理を記述します。
            //---------------------------------------------------------

            //---------------------------------------------------------
            //↑ユーザが記述するエラー処理はここまでです。
            //---------------------------------------------------------

            return inReturn;
        }
    }
}
