﻿using Eclr;
using Eclr.Pcos;
using ProConOS_eCLR;
using RTC_APILib;
using RTC_CAN_Lib;
using RTC_COMLib;
using RTC_ECAT_Lib;
using RTC_LOGLib;
using RTC_SOCKLib;
using System;
using System.Iec61131Lib;


namespace RTC_Template
{
    /// <summary>
    /// RT-C共通処理クラス
    /// </summary>
    /// 

    public unsafe class CRTC
    {
        //-----------------------------------
        //■ステータス定義
        //-----------------------------------
        public const short STS_INI = 0;    //初期処理
        public const short STS_PRO = 1;    //通常処理
        public const short STS_END = 2;    //終了処理
        public const short STS_ERR = 3;    //エラー処理

        //-----------------------------------
        //■構造体定義
        //-----------------------------------

        //デバッグ用構造体（デバッグ時下記の構造体とデバッグツールの構造体を合わせる必要が有ります）
        public struct M_AREA
        {
            public int inSystemCounter;		//システム時間カウンタ、システム時間毎に増加
            //---------------------------------------------------------
            //↓以下、ユーザが構造体定義を記述します。
            //---------------------------------------------------------
            public int inC_MajorVersion;    //メジャーバージョン
            public int inC_MinorVersion;    //マイナーバージョン
            public int inC_BuildVersion;    //ビルドバージョン

            public int Array_0_0;           //サンプル　2次元配列データ[0][0]
            public int Array_9_9;           //サンプル　2次元配列データ[9][9]
            //---------------------------------------------------------
            //↑ユーザが記述する構造体定義はここまでです。
            //---------------------------------------------------------
        }

        //-----------------------------------
        //■変数定義
        //-----------------------------------

        //M_AREA共有メモリポインタ変数
        public static M_AREA* pMArea;

        //メモリ取得
        public static RtMemory rmData;  //共有メモリ
        public static int* pData;  //ポインタ変数

        /// <summary>
        /// コンストラクタ
        /// </summary>
        static CRTC()
        {
            //-----------------------------------
            //■デバッグ用メモリポインタ取得
            //-----------------------------------

            //デバッグ用メモリポインタ取得
            pMArea = (M_AREA*)cSharedArea.GetMemPtr();
            pMArea->inSystemCounter = 0;    //システムカウンタ

            //-----------------------------------
            //C#プログラムバージョン
            //-----------------------------------
            pMArea->inC_MajorVersion = 1;   //メジャーバージョン
            pMArea->inC_MinorVersion = 2;   //マイナーバージョン
            pMArea->inC_BuildVersion = 3;   //ビルドバージョン

            //-----------------------------------
            //■共有メモリ取得
            //-----------------------------------
            IecString80 sMemName = new IecString80();
            sMemName.ctor();
            sMemName.s.Init("RTC_Data");    //共有メモリ名（最大12文字）

            uint SharedMemSize = 4096 * 10; //共有メモリサイズ(4096Byte単位)　例：4096 * 10 = 40KB

            rmData = new RtMemory(SharedMemSize, (byte)HandleSelection.ROOT_PROCESS, sMemName);　//メモリ領域を確保
            if (rmData.GetLastErr() != 0)
            {
                //既に共有メモリが作成済みの場合
                IecString80 sProcName = new IecString80();
                sProcName.ctor();
                sProcName.s.Init("");

                rmData = new RtMemory(sProcName, sMemName); //既にある共有メモリを使用
                if (rmData.GetLastErr() != 0)
                {
                    //共有メモリ取得エラー
                    return;
                }
            }

            pData = (int*)rmData.GetMemPtr(); //共有メモリポインタ取得
        }

        /// <summary>
        /// デストラクタ
        /// </summary>
        ~CRTC()
        {
            //共有メモリが有効でであれば解放
            if (rmData != null)
            {
                rmData.Delete();    //データ格納領域を解放する
                rmData = null;      //変数初期化
            }
        }

    }


}
